==> nxui-0.14.1.crate: nxui-0.14.1/src/chooser.rs <==
use std::ffi::OsStr;
use std::iter::once;
use std::ptr::null_mut;
use std::mem::size_of;
use std::result;


#[cfg(windows)]
const SZW_SIZE: usize = 512;
#[cfg(windows)]
type Result<T> = result::Result<T,String>;
#[cfg(windows)]
type TSzwBuf   = [u16; SZW_SIZE];


#[cfg(windows)]
pub struct FileChooser<'a> {
    pub title: &'a str
}

#[cfg(windows)]
impl<'a> FileChooser<'a> {
    pub fn new(title: &'a str) -> Self {
        Self {
            title
        }
    }

    #[allow(non_snake_case)]
    pub fn show(&self) -> Result<String> {
        use winapi::um::commdlg::{OPENFILENAMEW, OFN_HIDEREADONLY};
        use winapi::um::commdlg::GetOpenFileNameW;

        const SZW_SIZE: usize = 512;
        type TSzwBuf   = [u16; SZW_SIZE];

        let szwFilter = str_to_szw(" ");
        let szwTitle  = str_to_szw(self.title);
        let mut szw_buf: TSzwBuf = [0; SZW_SIZE];
        let mut dlgOpen = OPENFILENAMEW {
            lStructSize:       size_of::<OPENFILENAMEW>() as u32, //DWORD,
            hwndOwner:         null_mut(),                        // HWND,
            hInstance:         null_mut(),                        // HINSTANCE,
            lpstrFilter:       szwFilter.as_ptr(),                // LPCWSTR,
            lpstrCustomFilter: null_mut(),                        // LPWSTR,
            nMaxCustFilter:    0,                                 // DWORD,
            nFilterIndex:      0,                                 // DWORD,
            lpstrFile:         szw_buf.as_mut_ptr(),              // LPWSTR,
            nMaxFile:          szw_buf.len() as u32,              // DWORD,
            lpstrFileTitle:    null_mut(),                        // LPWSTR,
            nMaxFileTitle:     0,                                 // DWORD,
            lpstrInitialDir:   null_mut(),                        // LPCWSTR,
            lpstrTitle:        szwTitle.as_ptr(),                 // LPCWSTR,
            Flags:             OFN_HIDEREADONLY,                  // DWORD,
            nFileOffset:       0,                                 // WORD,
            nFileExtension:    0,                                 // WORD,
            lpstrDefExt:       null_mut(),                        // LPCWSTR,
            lCustData:         0,                                 // LPARAM,
            lpfnHook:          None,                              // LPOFNHOOKPROC,
            lpTemplateName:    null_mut(),                        // LPCWSTR,
            pvReserved:        null_mut(),                        // *mut c_void,
            dwReserved:        0,                                 // DWORD,
            FlagsEx:           0,                                 // DWORD,
        };

        match unsafe { GetOpenFileNameW(&mut dlgOpen) } {
            0 => Err("Nothing is selected !".to_string()),
            _ => szw_to_string( &szw_buf ),
        }
    }
}

#[cfg(windows)]
fn szw_to_string( szwbuf: &TSzwBuf ) -> Result<String> {
    szwbuf.iter()
        .position(|wch| wch == &0)
        .ok_or("String : Can't find zero terminator !".to_owned())
        .and_then(|ix| String::from_utf16( &szwbuf[..ix] )
            .map_err(|e| e.to_string()))
}

#[cfg(windows)]
fn str_to_szw(str_body: &str) -> Vec<u16> {
    use std::os::windows::ffi::OsStrExt;
    return OsStr::new(str_body)
        .encode_wide()
        .chain(once(0))
        .collect::<Vec<u16>>();
}

#[cfg(unix)]
pub struct FileChooser {

}

#[cfg(unix)]
impl FileChooser {

}

==> nxui-0.14.3.crate: nxui-0.14.3/src/chooser.rs <==
//! Displays a dialog to select a file

use std::ffi::OsStr;
use std::iter::once;
use std::ptr::null_mut;
use std::mem::size_of;
use std::result;


#[cfg(windows)]
const SZW_SIZE: usize = 512;
#[cfg(windows)]
type Result<T> = result::Result<T,String>;
#[cfg(windows)]
type TSzwBuf   = [u16; SZW_SIZE];


#[cfg(windows)]
pub struct FileChooser<'a> {
    /// Dialog Title
    pub title: &'a str
}

#[cfg(windows)]
impl<'a> FileChooser<'a> {
    pub fn new(title: &'a str) -> Self {
        Self {
            title
        }
    }

    #[allow(non_snake_case)]
    pub fn show(&self) -> String {
        use winapi::um::commdlg::{OPENFILENAMEW, OFN_HIDEREADONLY};
        use winapi::um::commdlg::GetOpenFileNameW;

        const SZW_SIZE: usize = 512;
        type TSzwBuf   = [u16; SZW_SIZE];

        let szwFilter = str_to_szw(" ");
        let szwTitle  = str_to_szw(self.title);
        let mut szw_buf: TSzwBuf = [0; SZW_SIZE];
        let mut dlgOpen = OPENFILENAMEW {
            lStructSize:       size_of::<OPENFILENAMEW>() as u32, //DWORD,
            hwndOwner:         null_mut(),                        // HWND,
            hInstance:         null_mut(),                        // HINSTANCE,
            lpstrFilter:       szwFilter.as_ptr(),                // LPCWSTR,
            lpstrCustomFilter: null_mut(),                        // LPWSTR,
            nMaxCustFilter:    0,                                 // DWORD,
            nFilterIndex:      0,                                 // DWORD,
            lpstrFile:         szw_buf.as_mut_ptr(),              // LPWSTR,
            nMaxFile:          szw_buf.len() as u32,              // DWORD,
            lpstrFileTitle:    null_mut(),                        // LPWSTR,
            nMaxFileTitle:     0,                                 // DWORD,
            lpstrInitialDir:   null_mut(),                        // LPCWSTR,
            lpstrTitle:        szwTitle.as_ptr(),                 // LPCWSTR,
            Flags:             OFN_HIDEREADONLY,                  // DWORD,
            nFileOffset:       0,                                 // WORD,
            nFileExtension:    0,                                 // WORD,
            lpstrDefExt:       null_mut(),                        // LPCWSTR,
            lCustData:         0,                                 // LPARAM,
            lpfnHook:          None,                              // LPOFNHOOKPROC,
            lpTemplateName:    null_mut(),                        // LPCWSTR,
            pvReserved:        null_mut(),                        // *mut c_void,
            dwReserved:        0,                                 // DWORD,
            FlagsEx:           0,                                 // DWORD,
        };

        let mut result = "".to_string();

        match unsafe { GetOpenFileNameW(&mut dlgOpen) } {
            0 => result = "Nothing is selected !".to_string(),
            _ => result = szw_to_string( &szw_buf ).unwrap(),
        };

        result
    }
}

#[cfg(windows)]
fn szw_to_string( szwbuf: &TSzwBuf ) -> Result<String> {
    szwbuf.iter()
        .position(|wch| wch == &0)
        .ok_or("String : Can't find zero terminator !".to_owned())
        .and_then(|ix| String::from_utf16( &szwbuf[..ix] )
            .map_err(|e| e.to_string()))
}

#[cfg(windows)]
fn str_to_szw(str_body: &str) -> Vec<u16> {
    use std::os::windows::ffi::OsStrExt;
    return OsStr::new(str_body)
        .encode_wide()
        .chain(once(0))
        .collect::<Vec<u16>>();
}

#[cfg(unix)]
pub struct FileChooser<'a> {
    pub title: &'a str
}

#[cfg(unix)]
impl<'a> FileChooser<'a> {
    pub fn new(title: &'a str) -> Self {
        Self {
            /// Dialog Title
			title
		}
    }

    pub fn show(&self) -> String {
        use gtk::prelude::{DialogExt, WidgetExt, WidgetExtManual};
        let dialog = gtk::FileChooserDialog::new::<gtk::Window>(Some("Open File"),
                                                                None,
                                                                gtk::FileChooserAction::Open);

        dialog.add_button("Cancel", gtk::ResponseType::Cancel.into());
        dialog.add_button("Open", gtk::ResponseType::Accept.into());

        let mut file = "".to_string();

        if dialog.run() == gtk::ResponseType::Accept.into() {
            if let path = dialog.path() {
                file = dialog.path().to_string()
            } else {
                file = "".to_string()
            }
        } else {
            file = "".to_string()
        }

        unsafe { dialog.destroy(); }

        file
    }
}
